@@ -245,6 +245,9 @@ json QgsJsonExporter::exportFeaturesToJsonObject( const QgsFeatureList &features
     { "type", "FeatureCollection" },
     { "features", json::array() }
   };
+
+  QgsJsonUtils::addCrsInfo( data, mDestinationCrs );
+
   for ( const QgsFeature &feature : std::as_const( features ) )
   {
     data["features"].push_back( exportFeatureToJsonObject( feature ) );
@@ -283,25 +286,25 @@ QString QgsJsonUtils::encodeValue( const QVariant &value )
   if ( QgsVariantUtils::isNull( value ) )
     return QStringLiteral( "null" );
 
-  switch ( value.type() )
+  switch ( value.userType() )
   {
-    case QVariant::Int:
-    case QVariant::UInt:
-    case QVariant::LongLong:
-    case QVariant::ULongLong:
-    case QVariant::Double:
+    case QMetaType::Type::Int:
+    case QMetaType::Type::UInt:
+    case QMetaType::Type::LongLong:
+    case QMetaType::Type::ULongLong:
+    case QMetaType::Type::Double:
       return value.toString();
 
-    case QVariant::Bool:
+    case QMetaType::Type::Bool:
       return value.toBool() ? "true" : "false";
 
-    case QVariant::StringList:
-    case QVariant::List:
-    case QVariant::Map:
+    case QMetaType::Type::QStringList:
+    case QMetaType::Type::QVariantList:
+    case QMetaType::Type::QVariantMap:
       return QString::fromUtf8( QJsonDocument::fromVariant( value ).toJson( QJsonDocument::Compact ) );
 
     default:
-    case QVariant::String:
+    case QMetaType::Type::QString:
       QString v = value.toString()
                   .replace( '\\', QLatin1String( "\\\\" ) )
                   .replace( '"', QLatin1String( "\\\"" ) )
@@ -339,7 +342,7 @@ QString QgsJsonUtils::exportAttributes( const QgsFeature &feature, QgsVectorLaye
   return attrs.prepend( '{' ).append( '}' );
 }
 
-QVariantList QgsJsonUtils::parseArray( const QString &json, QVariant::Type type )
+QVariantList QgsJsonUtils::parseArray( const QString &json, QMetaType::Type type )
 {
   QString errorMessage;
   QVariantList result;
@@ -378,11 +381,11 @@ QVariantList QgsJsonUtils::parseArray( const QString &json, QVariant::Type type
       else if ( item.is_null() )
       {
         // Fallback to int
-        v = QVariant( type == QVariant::Type::Invalid ? QVariant::Type::Int : type );
+        v = QgsVariantUtils::createNullVariant( type == QMetaType::Type::UnknownType ? QMetaType::Type::Int : type );
       }
 
       // If a destination type was specified (it's not invalid), try to convert
-      if ( type != QVariant::Invalid )
+      if ( type != QMetaType::Type::UnknownType )
       {
         if ( ! v.convert( static_cast<int>( type ) ) )
         {
@@ -408,6 +411,11 @@ QVariantList QgsJsonUtils::parseArray( const QString &json, QVariant::Type type
   return result;
 }
 
+QVariantList QgsJsonUtils::parseArray( const QString &json, QVariant::Type type )
+{
+  return parseArray( json, QgsVariantUtils::variantTypeToMetaType( type ) );
+}
+
 std::unique_ptr< QgsPoint> parsePointFromGeoJson( const json &coords )
 {
   if ( !coords.is_array() || coords.size() < 2 || coords.size() > 3 )
@@ -718,7 +726,7 @@ json QgsJsonUtils::jsonFromVariant( const QVariant &val )
     return nullptr;
   }
   json j;
-  if ( val.type() == QVariant::Type::Map )
+  if ( val.userType() == QMetaType::Type::QVariantMap )
   {
     const QVariantMap &vMap = val.toMap();
     json jMap = json::object();
@@ -728,7 +736,7 @@ json QgsJsonUtils::jsonFromVariant( const QVariant &val )
     }
     j = jMap;
   }
-  else if ( val.type() == QVariant::Type::List || val.type() == QVariant::Type::StringList )
+  else if ( val.userType() == QMetaType::Type::QVariantList || val.userType() == QMetaType::Type::QStringList )
   {
     const QVariantList &vList = val.toList();
     json jList = json::array();
@@ -912,3 +920,14 @@ json QgsJsonUtils::exportAttributesToJsonObject( const QgsFeature &feature, QgsV
   }
   return attrs;
 }
+
+void QgsJsonUtils::addCrsInfo( json &value, const QgsCoordinateReferenceSystem &crs )
+{
+  // When user request EPSG:4326 we return a compliant CRS84 lon/lat GeoJSON
+  // so no need to add CRS information
+  if ( crs.authid() == "OGC:CRS84" || crs.authid() == "EPSG:4326" )
+    return;
+
+  value["crs"]["type"] = "name";
+  value["crs"]["properties"]["name"] = crs.toOgcUrn().toStdString();
+}
