@@ -51,6 +51,7 @@
 #include "qgspointcloudlayer.h"
 #include "qgspointcloudrenderer.h"
 #include "qgspointcloudlayerelevationproperties.h"
+#include "qgsrasterlayerelevationproperties.h"
 #include "qgssymbol.h"
 #include "qgsguiutils.h"
 
@@ -213,30 +214,51 @@ bool QgsMapToolIdentify::identifyLayer( QList<IdentifyResult> *results, QgsMapLa
 
 bool QgsMapToolIdentify::identifyLayer( QList<IdentifyResult> *results, QgsMapLayer *layer, const QgsGeometry &geometry, const QgsRectangle &viewExtent, double mapUnitsPerPixel, QgsMapToolIdentify::LayerType layerType, const QgsIdentifyContext &identifyContext )
 {
-  if ( layer->type() == Qgis::LayerType::Raster && layerType.testFlag( RasterLayer ) )
+  switch ( layer->type() )
   {
-    return identifyRasterLayer( results, qobject_cast<QgsRasterLayer *>( layer ), geometry, viewExtent, mapUnitsPerPixel, identifyContext );
-  }
-  else if ( layer->type() == Qgis::LayerType::Vector && layerType.testFlag( VectorLayer ) )
-  {
-    return identifyVectorLayer( results, qobject_cast<QgsVectorLayer *>( layer ), geometry, identifyContext );
-  }
-  else if ( layer->type() == Qgis::LayerType::Mesh && layerType.testFlag( MeshLayer ) )
-  {
-    return identifyMeshLayer( results, qobject_cast<QgsMeshLayer *>( layer ), geometry, identifyContext );
-  }
-  else if ( layer->type() == Qgis::LayerType::VectorTile && layerType.testFlag( VectorTileLayer ) )
-  {
-    return identifyVectorTileLayer( results, qobject_cast<QgsVectorTileLayer *>( layer ), geometry, identifyContext );
-  }
-  else if ( layer->type() == Qgis::LayerType::PointCloud && layerType.testFlag( PointCloudLayer ) )
-  {
-    return identifyPointCloudLayer( results, qobject_cast<QgsPointCloudLayer *>( layer ), geometry, identifyContext );
-  }
-  else
-  {
-    return false;
+    case Qgis::LayerType::Vector:
+      if ( layerType.testFlag( VectorLayer ) )
+      {
+        return identifyVectorLayer( results, qobject_cast<QgsVectorLayer *>( layer ), geometry, identifyContext );
+      }
+      break;
+
+    case Qgis::LayerType::Raster:
+      if ( layerType.testFlag( RasterLayer ) )
+      {
+        return identifyRasterLayer( results, qobject_cast<QgsRasterLayer *>( layer ), geometry, viewExtent, mapUnitsPerPixel, identifyContext );
+      }
+      break;
+
+    case Qgis::LayerType::Mesh:
+      if ( layerType.testFlag( MeshLayer ) )
+      {
+        return identifyMeshLayer( results, qobject_cast<QgsMeshLayer *>( layer ), geometry, identifyContext );
+      }
+      break;
+
+    case Qgis::LayerType::VectorTile:
+      if ( layerType.testFlag( VectorTileLayer ) )
+      {
+        return identifyVectorTileLayer( results, qobject_cast<QgsVectorTileLayer *>( layer ), geometry, identifyContext );
+      }
+      break;
+
+    case Qgis::LayerType::PointCloud:
+      if ( layerType.testFlag( PointCloudLayer ) )
+      {
+        return identifyPointCloudLayer( results, qobject_cast<QgsPointCloudLayer *>( layer ), geometry, identifyContext );
+      }
+      break;
+
+    // not supported
+    case Qgis::LayerType::Plugin:
+    case Qgis::LayerType::Annotation:
+    case Qgis::LayerType::Group:
+    case Qgis::LayerType::TiledScene:
+      break;
   }
+  return false;
 }
 
 bool QgsMapToolIdentify::identifyVectorLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsVectorLayer *layer, const QgsPointXY &point, const QgsIdentifyContext &identifyContext )
@@ -256,6 +278,12 @@ bool QgsMapToolIdentify::identifyMeshLayer( QList<QgsMapToolIdentify::IdentifyRe
   if ( !layer )
     return false;
 
+  if ( !identifyContext.zRange().isInfinite() )
+  {
+    if ( !layer->elevationProperties()->isVisibleInZRange( identifyContext.zRange() ) )
+      return false;
+  }
+
   double searchRadius = mOverrideCanvasSearchRadius < 0 ? searchRadiusMU( mCanvas ) : mOverrideCanvasSearchRadius;
   bool isTemporal = identifyContext.isTemporal() && layer->temporalProperties()->isActive();
 
@@ -264,7 +292,7 @@ bool QgsMapToolIdentify::identifyMeshLayer( QList<QgsMapToolIdentify::IdentifyRe
   int activeVectorGroup = layer->rendererSettings().activeVectorDatasetGroup();
 
   const QList<int> allGroup = layer->enabledDatasetGroupsIndexes();
-  if ( isTemporal ) //non active dataset group value are only accesible if temporal is active
+  if ( isTemporal ) //non active dataset group value are only accessible if temporal is active
   {
     const QgsDateTimeRange &time = identifyContext.temporalRange();
     if ( activeScalarGroup >= 0 )
@@ -510,11 +538,18 @@ bool QgsMapToolIdentify::identifyVectorTileLayer( QList<QgsMapToolIdentify::Iden
 
 bool QgsMapToolIdentify::identifyPointCloudLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsPointCloudLayer *layer, const QgsGeometry &geometry, const QgsIdentifyContext &identifyContext )
 {
-  Q_UNUSED( identifyContext )
+  if ( !identifyContext.zRange().isInfinite() )
+  {
+    if ( !layer->elevationProperties()->isVisibleInZRange( identifyContext.zRange(), layer ) )
+      return false;
+  }
+
   QgsPointCloudRenderer *renderer = layer->renderer();
 
   QgsRenderContext context = QgsRenderContext::fromMapSettings( mCanvas->mapSettings() );
   context.setCoordinateTransform( QgsCoordinateTransform( layer->crs(), mCanvas->mapSettings().destinationCrs(), mCanvas->mapSettings().transformContext() ) );
+  if ( !identifyContext.zRange().isInfinite() )
+    context.setZRange( identifyContext.zRange() );
 
   const double searchRadiusMapUnits = mOverrideCanvasSearchRadius < 0 ? searchRadiusMU( mCanvas ) : mOverrideCanvasSearchRadius;
 
@@ -959,8 +994,8 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
   if ( !dprovider )
     return false;
 
-  int capabilities = dprovider->capabilities();
-  if ( !( capabilities & QgsRasterDataProvider::Identify ) )
+  const Qgis::RasterInterfaceCapabilities capabilities = dprovider->capabilities();
+  if ( !( capabilities & Qgis::RasterInterfaceCapability::Identify ) )
     return false;
 
   if ( identifyContext.isTemporal() )
@@ -971,6 +1006,12 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
     dprovider->temporalCapabilities()->setRequestedTemporalRange( identifyContext.temporalRange() );
   }
 
+  if ( !identifyContext.zRange().isInfinite() )
+  {
+    if ( !layer->elevationProperties()->isVisibleInZRange( identifyContext.zRange(), layer ) )
+      return false;
+  }
+
   QgsPointXY pointInCanvasCrs = point;
   try
   {
@@ -992,15 +1033,15 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
   Qgis::RasterIdentifyFormat format = QgsRasterDataProvider::identifyFormatFromName( layer->customProperty( QStringLiteral( "identify/format" ) ).toString() );
 
   // check if the format is really supported otherwise use first supported format
-  if ( !( QgsRasterDataProvider::identifyFormatToCapability( format ) & capabilities ) )
+  if ( !( capabilities & QgsRasterDataProvider::identifyFormatToCapability( format ) ) )
   {
-    if ( capabilities & QgsRasterInterface::IdentifyFeature )
+    if ( capabilities & Qgis::RasterInterfaceCapability::IdentifyFeature )
       format = Qgis::RasterIdentifyFormat::Feature;
-    else if ( capabilities & QgsRasterInterface::IdentifyValue )
+    else if ( capabilities & Qgis::RasterInterfaceCapability::IdentifyValue )
       format = Qgis::RasterIdentifyFormat::Value;
-    else if ( capabilities & QgsRasterInterface::IdentifyHtml )
+    else if ( capabilities & Qgis::RasterInterfaceCapability::IdentifyHtml )
       format = Qgis::RasterIdentifyFormat::Html;
-    else if ( capabilities & QgsRasterInterface::IdentifyText )
+    else if ( capabilities & Qgis::RasterInterfaceCapability::IdentifyText )
       format = Qgis::RasterIdentifyFormat::Text;
     else return false;
   }
@@ -1049,13 +1090,56 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
     identifyResult = dprovider->identify( point, format, viewExtent, width, height );
   }
 
+  QgsRasterLayerElevationProperties *elevationProperties = qobject_cast< QgsRasterLayerElevationProperties *>( layer->elevationProperties() );
+  if ( identifyResult.isValid() && !identifyContext.zRange().isInfinite() && elevationProperties && elevationProperties->isEnabled() )
+  {
+    // filter results by z range
+    switch ( format )
+    {
+      case Qgis::RasterIdentifyFormat::Value:
+      {
+        bool foundMatch = false;
+        QMap<int, QVariant> values = identifyResult.results();
+        QMap<int, QVariant> filteredValues;
+        for ( auto it = values.constBegin(); it != values.constEnd(); ++it )
+        {
+          if ( QgsVariantUtils::isNull( it.value() ) )
+          {
+            continue;
+          }
+          const double value = it.value().toDouble();
+          const QgsDoubleRange elevationRange = elevationProperties->elevationRangeForPixelValue( layer, it.key(), value );
+          if ( !elevationRange.isInfinite() && identifyContext.zRange().overlaps( elevationRange ) )
+          {
+            filteredValues.insert( it.key(), it.value() );
+            foundMatch = true;
+          }
+        }
+
+        if ( !foundMatch )
+          return false;
+
+        identifyResult = QgsRasterIdentifyResult( Qgis::RasterIdentifyFormat::Value, filteredValues );
+
+        break;
+      }
+
+      // can't filter by z for these formats
+      case Qgis::RasterIdentifyFormat::Undefined:
+      case Qgis::RasterIdentifyFormat::Text:
+      case Qgis::RasterIdentifyFormat::Html:
+      case Qgis::RasterIdentifyFormat::Feature:
+        break;
+    }
+  }
+
   derivedAttributes.insert( derivedAttributesForPoint( QgsPoint( pointInCanvasCrs ) ) );
 
   const double xres = layer->rasterUnitsPerPixelX();
   const double yres = layer->rasterUnitsPerPixelY();
   QgsRectangle pixelRect;
   // Don't derive clicked column/row for providers that serve dynamically rendered map images
-  if ( ( dprovider->capabilities() & QgsRasterDataProvider::Size ) && !qgsDoubleNear( xres, 0 ) && !qgsDoubleNear( yres, 0 ) )
+  if ( ( dprovider->capabilities() & Qgis::RasterInterfaceCapability::Size ) && !qgsDoubleNear( xres, 0 ) && !qgsDoubleNear( yres, 0 ) )
   {
     // Try to determine the clicked column/row (0-based) in the raster
     const QgsRectangle extent = dprovider->extent();
@@ -1090,13 +1174,13 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
           // The cast is legit. Quoting QT doc :
           // "Although this function is declared as returning QVariant::Type,
           // the return value should be interpreted as QMetaType::Type"
-          if ( static_cast<QMetaType::Type>( value.type() ) == QMetaType::Float )
+          if ( static_cast<QMetaType::Type>( value.userType() ) == QMetaType::Float )
           {
-            valueString = QgsRasterBlock::printValue( value.toFloat() );
+            valueString = QgsRasterBlock::printValue( value.toFloat(), true );
           }
           else
           {
-            valueString = QgsRasterBlock::printValue( value.toDouble() );
+            valueString = QgsRasterBlock::printValue( value.toDouble(), true );
           }
         }
         attributes.insert( dprovider->generateBandName( it.key() ), valueString );
@@ -1124,14 +1208,14 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
                 QString ratValue;
                 switch ( ratField.type )
                 {
-                  case QVariant::Type::Char:
-                  case QVariant::Type::Int:
-                  case QVariant::Type::UInt:
-                  case QVariant::Type::LongLong:
-                  case QVariant::Type::ULongLong:
+                  case QMetaType::Type::QChar:
+                  case QMetaType::Type::Int:
+                  case QMetaType::Type::UInt:
+                  case QMetaType::Type::LongLong:
+                  case QMetaType::Type::ULongLong:
                     ratValue = QLocale().toString( row.at( colIdx ).toLongLong() );
                     break;
-                  case QVariant::Type::Double:
+                  case QMetaType::Type::Double:
                     ratValue = QLocale().toString( row.at( colIdx ).toDouble( ) );
                     break;
                   default:
@@ -1161,13 +1245,13 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
       for ( auto it = values.constBegin(); it != values.constEnd(); ++it )
       {
         QVariant value = it.value();
-        if ( value.type() == QVariant::Bool && !value.toBool() )
+        if ( value.userType() == QMetaType::Type::Bool && !value.toBool() )
         {
           // sublayer not visible or not queryable
           continue;
         }
 
-        if ( value.type() == QVariant::String )
+        if ( value.userType() == QMetaType::Type::QString )
         {
           // error
           // TODO: better error reporting
