@@ -56,8 +56,6 @@
 #include <QDir>
 #include <QSignalSpy>
 
-class QgsCameraController4Test;
-
 class TestQgs3DRendering : public QgsTest
 {
     Q_OBJECT
@@ -88,6 +86,7 @@ class TestQgs3DRendering : public QgsTest
     void testBufferedLineRenderingWidth();
     void testMapTheme();
     void testRuleBasedRenderer();
+    void testFilteredRuleBasedRenderer();
     void testAnimationExport();
     void testBillboardRendering();
     void testInstancedRendering();
@@ -100,7 +99,7 @@ class TestQgs3DRendering : public QgsTest
     void test3DSceneExporter();
 
   private:
-    QImage convertDepthImageToGray16Image( const QImage &depthImage );
+    QImage convertDepthImageToGrayscaleImage( const QImage &depthImage );
 
     void do3DSceneExport( int zoomLevelsCount, int expectedObjectCount, int maxFaceCount, Qgs3DMapScene *scene, QgsPolygon3DSymbol *symbol3d,
                           QgsVectorLayer *layerPoly, QgsOffscreen3DEngine *engine );
@@ -111,57 +110,39 @@ class TestQgs3DRendering : public QgsTest
     QgsVectorLayer *mLayerBuildings = nullptr;
 };
 
-/**
- * \ingroup UnitTests
- * Helper class to access QgsCameraController properties
- */
-class QgsCameraController4Test : public QgsCameraController
+QImage TestQgs3DRendering::convertDepthImageToGrayscaleImage( const QImage &depthImage )
 {
-    Q_OBJECT
-  public:
-    QgsCameraController4Test( Qgs3DMapScene *parent = nullptr )
-      : QgsCameraController( parent )
-    { }
-
-    // wraps protected methods
-    void superOnWheel( Qt3DInput::QWheelEvent *wheel ) { onWheel( wheel ); }
-    void superOnMousePressed( Qt3DInput::QMouseEvent *mouse ) { onMousePressed( mouse ); }
-    double superSampleDepthBuffer( const QImage &buffer, int px, int py ) { return sampleDepthBuffer( buffer, px, py ); }
-
-    // wraps protected member vars
-    QVector3D zoomPoint() { return mZoomPoint; }
-    double cumulatedWheelY() { return mCumulatedWheelY; }
-    Qt3DRender::QCamera *cameraBefore() { return mCameraBefore.get(); }
-    QgsCameraPose *cameraPose() { return &mCameraPose; }
-};
-
-QImage TestQgs3DRendering::convertDepthImageToGray16Image( const QImage &depthImage )
-{
-  QImage grayImage( depthImage.width(), depthImage.height(), QImage::Format_Grayscale16 );
+  const int width = depthImage.width();
+  const int height = depthImage.height();
+  QImage grayImage( width, height, QImage::Format_ARGB32 );
 
   // compute image min/max depth values
   double minV = 9999999.0;
   double maxV = -9999999.0;
-  for ( int x = 0; x < grayImage.width(); x++ )
+  for ( int y = 0; y < height; ++y )
   {
-    for ( int y = 0; y < grayImage.height(); y++ )
+    const QRgb *depthImageScanline = reinterpret_cast< const QRgb * >( depthImage.scanLine( y ) );
+    for ( int x = 0; x < width; ++x )
     {
-      double d = Qgs3DUtils::decodeDepth( depthImage.pixel( x, y ) );
-      if ( d > maxV ) maxV = d;
-      else if ( d < minV ) minV = d;
+      const double d = Qgs3DUtils::decodeDepth( depthImageScanline[x] );
+      if ( d > maxV )
+        maxV = d;
+      if ( d < minV )
+        minV = d;
     }
   }
 
   // transform depth value to gray value
-  double factor = 65635.0 / ( maxV - minV );
-  for ( int x = 0; x < grayImage.width(); x++ )
+  const double factor = ( maxV > minV ) ? 255.0 / ( maxV - minV ) : 1.0;
+  for ( int y = 0; y < height; ++y )
   {
-    for ( int y = 0; y < grayImage.height(); y++ )
+    const QRgb *depthImageScanline = reinterpret_cast< const QRgb * >( depthImage.scanLine( y ) );
+    QRgb *grayImageScanline = reinterpret_cast< QRgb * >( grayImage.scanLine( y ) );
+    for ( int x = 0; x < width; ++x )
     {
-      double d = Qgs3DUtils::decodeDepth( depthImage.pixel( x, y ) );
-      unsigned short v = ( unsigned short )( factor * ( d - minV ) );
-      QRgba64 col = QRgba64::fromRgba64( v, v, v, ( quint16 )65635 );
-      grayImage.setPixelColor( x, y, QColor( col ) );
+      const double d = Qgs3DUtils::decodeDepth( depthImageScanline[x] );
+      unsigned short v = static_cast< unsigned short >( factor * ( d - minV ) );
+      grayImageScanline[x] = qRgb( v, v, v );
     }
   }
 
@@ -1061,6 +1042,66 @@ void TestQgs3DRendering::testRuleBasedRenderer()
   QGSVERIFYIMAGECHECK( "rulebased", "rulebased", img, QString(), 40, QSize( 0, 0 ), 2 );
 }
 
+void TestQgs3DRendering::testFilteredRuleBasedRenderer()
+{
+  QgsPhongMaterialSettings materialSettings;
+  materialSettings.setAmbient( Qt::lightGray );
+  QgsPolygon3DSymbol *symbol3d = new QgsPolygon3DSymbol;
+  symbol3d->setMaterialSettings( materialSettings.clone() );
+  symbol3d->setExtrusionHeight( 10.f );
+
+  QgsPhongMaterialSettings materialSettings2;
+  materialSettings2.setAmbient( Qt::red );
+  QgsPolygon3DSymbol *symbol3d2 = new QgsPolygon3DSymbol;
+  symbol3d2->setMaterialSettings( materialSettings2.clone() );
+  symbol3d2->setExtrusionHeight( 10.f );
+
+  QgsRuleBased3DRenderer::Rule *root = new QgsRuleBased3DRenderer::Rule( nullptr );
+  QgsRuleBased3DRenderer::Rule *rule1 = new QgsRuleBased3DRenderer::Rule( symbol3d, "ogc_fid < 29069", "rule 1" );
+  QgsRuleBased3DRenderer::Rule *rule2 = new QgsRuleBased3DRenderer::Rule( symbol3d2, "ogc_fid >= 29069", "rule 2" );
+  root->appendChild( rule1 );
+  root->appendChild( rule2 );
+  QgsRuleBased3DRenderer *renderer3d = new QgsRuleBased3DRenderer( root );
+  mLayerBuildings->setRenderer3D( renderer3d );
+
+  QgsRectangle fullExtent = mLayerBuildings->extent();
+  const QgsRectangle filteredExtent = QgsRectangle( fullExtent.xMinimum(), fullExtent.yMinimum(),
+                                      fullExtent.xMinimum() + 0.8 * fullExtent.width(), fullExtent.yMinimum() + 0.7 * fullExtent.height() );
+
+  Qgs3DMapSettings *map = new Qgs3DMapSettings;
+  map->setCrs( mProject->crs() );
+  map->setExtent( filteredExtent );
+  map->setLayers( QList<QgsMapLayer *>() << mLayerBuildings );
+
+  QgsPointLightSettings defaultLight;
+  defaultLight.setIntensity( 0.5 );
+  defaultLight.setPosition( QgsVector3D( 0, 1000, 0 ) );
+  map->setLightSources( { defaultLight.clone() } );
+
+  QgsFlatTerrainGenerator *flatTerrain = new QgsFlatTerrainGenerator;
+  flatTerrain->setCrs( map->crs() );
+  map->setTerrainGenerator( flatTerrain );
+
+  QgsOffscreen3DEngine engine;
+  Qgs3DMapScene *scene = new Qgs3DMapScene( *map, &engine );
+  engine.setRootEntity( scene );
+
+  scene->cameraController()->setLookingAtPoint( QgsVector3D( 0, 0, 250 ), 500, 45, 0 );
+
+  // When running the test, it would sometimes return partially rendered image.
+  // It is probably based on how fast qt3d manages to upload the data to GPU...
+  // Capturing the initial image and throwing it away fixes that. Hopefully we will
+  // find a better fix in the future.
+  Qgs3DUtils::captureSceneImage( engine, scene );
+
+  QImage img = Qgs3DUtils::captureSceneImage( engine, scene );
+
+  delete scene;
+  delete map;
+
+  QGSVERIFYIMAGECHECK( "rulebased_filtered", "rulebased_filtered", img, QString(), 40, QSize( 0, 0 ), 2 );
+}
+
 void TestQgs3DRendering::testAnimationExport()
 {
   const QgsRectangle fullExtent = mLayerDtm->extent();
@@ -1533,95 +1574,94 @@ void TestQgs3DRendering::testDepthBuffer()
 
   // =============================================
   // =========== TEST DEPTH
-  QgsCameraController4Test *testCam = static_cast<QgsCameraController4Test *>( scene->cameraController() );
 
   // retrieve 3D depth image
   QImage depthImage = Qgs3DUtils::captureSceneDepthBuffer( engine, scene );
-  QImage grayImage = convertDepthImageToGray16Image( depthImage );
-  QGSVERIFYIMAGECHECK( "depth_retrieve_image", "depth_retrieve_image", grayImage, QString(), 550, QSize( 0, 0 ), 2 );
+  QImage grayImage = convertDepthImageToGrayscaleImage( depthImage );
+  QGSVERIFYIMAGECHECK( "depth_retrieve_image", "depth_retrieve_image", grayImage, QString(), 5, QSize( 0, 0 ), 2 );
 
   // =========== TEST WHEEL ZOOM
   QVector3D startPos = scene->cameraController()->camera()->position();
   // set cameraController mouse pos to middle screen
   QMouseEvent mouseEvent( QEvent::MouseButtonPress, QPointF( midPos.x(), midPos.y() ),
                           Qt::MiddleButton, Qt::MiddleButton, Qt::NoModifier );
-  testCam->superOnMousePressed( new Qt3DInput::QMouseEvent( mouseEvent ) );
+  scene->cameraController()->onMousePressed( new Qt3DInput::QMouseEvent( mouseEvent ) );
 
   // Check first wheel action
   QWheelEvent wheelEvent( midPos, midPos, QPoint(), QPoint( 0, 120 ),
                           Qt::NoButton, Qt::NoModifier, Qt::NoScrollPhase,
                           false, Qt::MouseEventSynthesizedByApplication );
-  testCam->superOnWheel( new Qt3DInput::QWheelEvent( wheelEvent ) );
-  QCOMPARE( testCam->cumulatedWheelY(), wheelEvent.angleDelta().y() * 5 );
-  QCOMPARE( testCam->cameraBefore()->viewCenter(), testCam->cameraPose()->centerPoint().toVector3D() );
+  scene->cameraController()->onWheel( new Qt3DInput::QWheelEvent( wheelEvent ) );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, wheelEvent.angleDelta().y() * 5 );
+  QCOMPARE( scene->cameraController()->mCameraBefore->viewCenter(), scene->cameraController()->cameraPose().centerPoint().toVector3D() );
 
   depthImage = Qgs3DUtils::captureSceneDepthBuffer( engine, scene );
-  grayImage = convertDepthImageToGray16Image( depthImage );
-  QGSVERIFYIMAGECHECK( "depth_wheel_action_1", "depth_wheel_action_1", grayImage, QString(), 550, QSize( 0, 0 ), 2 );
+  grayImage = convertDepthImageToGrayscaleImage( depthImage );
+  QGSVERIFYIMAGECHECK( "depth_wheel_action_1", "depth_wheel_action_1", grayImage, QString(), 5, QSize( 0, 0 ), 2 );
 
   scene->cameraController()->depthBufferCaptured( depthImage );
 
-  QGSCOMPARENEARVECTOR3D( testCam->zoomPoint(), QVector3D( -32.7, 224.6, 185.5 ), 1.0 );
-  QGSCOMPARENEARVECTOR3D( testCam->cameraPose()->centerPoint(), QVector3D( -32.7, 224.6, 185.5 ), 1.0 );
-  QGSCOMPARENEAR( testCam->cameraPose()->distanceFromCenterPoint(), 955.4, 1.0 );
-  QCOMPARE( testCam->cumulatedWheelY(), 0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->mZoomPoint, QVector3D( -32.7, 224.6, 185.5 ), 1.0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->cameraPose().centerPoint(), QVector3D( -32.7, 224.6, 185.5 ), 1.0 );
+  QGSCOMPARENEAR( scene->cameraController()->cameraPose().distanceFromCenterPoint(), 955.4, 1.0 );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, 0 );
 
   // Checking second wheel action
   QWheelEvent wheelEvent2( midPos, midPos, QPoint(), QPoint( 0, 120 ),
                            Qt::NoButton, Qt::NoModifier, Qt::NoScrollPhase,
                            false, Qt::MouseEventSynthesizedByApplication );
-  testCam->superOnWheel( new Qt3DInput::QWheelEvent( wheelEvent2 ) );
-  QCOMPARE( testCam->cumulatedWheelY(), wheelEvent2.angleDelta().y() * 5 );
-  QCOMPARE( testCam->cameraBefore()->viewCenter(), testCam->cameraPose()->centerPoint().toVector3D() );
+  scene->cameraController()->onWheel( new Qt3DInput::QWheelEvent( wheelEvent2 ) );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, wheelEvent2.angleDelta().y() * 5 );
+  QCOMPARE( scene->cameraController()->mCameraBefore->viewCenter(), scene->cameraController()->cameraPose().centerPoint().toVector3D() );
 
   depthImage = Qgs3DUtils::captureSceneDepthBuffer( engine, scene );
-  grayImage = convertDepthImageToGray16Image( depthImage );
-  QGSVERIFYIMAGECHECK( "depth_wheel_action_2", "depth_wheel_action_2", grayImage, QString(), 550, QSize( 0, 0 ), 2 );
+  grayImage = convertDepthImageToGrayscaleImage( depthImage );
+  QGSVERIFYIMAGECHECK( "depth_wheel_action_2", "depth_wheel_action_2", grayImage, QString(), 5, QSize( 0, 0 ), 2 );
 
   scene->cameraController()->depthBufferCaptured( depthImage );
 
-  QGSCOMPARENEARVECTOR3D( testCam->zoomPoint(), QVector3D( -32.5, 223.5, 184.7 ), 1.0 );
-  QGSCOMPARENEARVECTOR3D( testCam->cameraPose()->centerPoint(), QVector3D( -32.5, 223.5, 184.7 ), 1.0 );
-  QGSCOMPARENEAR( testCam->cameraPose()->distanceFromCenterPoint(), 757.4, 1.0 );
-  QCOMPARE( testCam->cumulatedWheelY(), 0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->mZoomPoint, QVector3D( -32.5, 223.5, 184.7 ), 1.0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->cameraPose().centerPoint(), QVector3D( -32.5, 223.5, 184.7 ), 1.0 );
+  QGSCOMPARENEAR( scene->cameraController()->cameraPose().distanceFromCenterPoint(), 757.4, 1.0 );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, 0 );
 
   // Checking third wheel action
   QWheelEvent wheelEvent3( midPos, midPos, QPoint( ), QPoint( 0, 480 ),
                            Qt::NoButton, Qt::NoModifier, Qt::NoScrollPhase,
                            false, Qt::MouseEventSynthesizedByApplication );
-  testCam->superOnWheel( new Qt3DInput::QWheelEvent( wheelEvent3 ) );
-  QCOMPARE( testCam->cumulatedWheelY(), wheelEvent3.angleDelta().y() * 5 );
-  QCOMPARE( testCam->cameraBefore()->viewCenter(), testCam->cameraPose()->centerPoint().toVector3D() );
+  scene->cameraController()->onWheel( new Qt3DInput::QWheelEvent( wheelEvent3 ) );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, wheelEvent3.angleDelta().y() * 5 );
+  QCOMPARE( scene->cameraController()->mCameraBefore->viewCenter(), scene->cameraController()->cameraPose().centerPoint().toVector3D() );
 
   depthImage = Qgs3DUtils::captureSceneDepthBuffer( engine, scene );
-  grayImage = convertDepthImageToGray16Image( depthImage );
-  QGSVERIFYIMAGECHECK( "depth_wheel_action_3", "depth_wheel_action_3", grayImage, QString(), 550, QSize( 0, 0 ), 2 );
+  grayImage = convertDepthImageToGrayscaleImage( depthImage );
+  QGSVERIFYIMAGECHECK( "depth_wheel_action_3", "depth_wheel_action_3", grayImage, QString(), 5, QSize( 0, 0 ), 2 );
 
   scene->cameraController()->depthBufferCaptured( depthImage );
 
-  QGSCOMPARENEARVECTOR3D( testCam->zoomPoint(), QVector3D( -32.4, 222.8, 184.1 ), 1.0 );
-  QGSCOMPARENEARVECTOR3D( testCam->cameraPose()->centerPoint(), QVector3D( -32.4, 222.8, 184.1 ), 1.0 );
-  QGSCOMPARENEAR( testCam->cameraPose()->distanceFromCenterPoint(), 126.4, 0.1 );
-  QCOMPARE( testCam->cumulatedWheelY(), 0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->mZoomPoint, QVector3D( -32.4, 222.8, 184.1 ), 1.0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->cameraPose().centerPoint(), QVector3D( -32.4, 222.8, 184.1 ), 1.0 );
+  QGSCOMPARENEAR( scene->cameraController()->cameraPose().distanceFromCenterPoint(), 126.4, 0.1 );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, 0 );
 
   // Checking fourth wheel action
   QWheelEvent wheelEvent4( midPos, midPos, QPoint(), QPoint( 0, 120 ),
                            Qt::NoButton, Qt::NoModifier, Qt::NoScrollPhase,
                            false, Qt::MouseEventSynthesizedByApplication );
-  testCam->superOnWheel( new Qt3DInput::QWheelEvent( wheelEvent4 ) );
-  QCOMPARE( testCam->cumulatedWheelY(), wheelEvent4.angleDelta().y() * 5 );
-  QCOMPARE( testCam->cameraBefore()->viewCenter(), testCam->cameraPose()->centerPoint().toVector3D() );
+  scene->cameraController()->onWheel( new Qt3DInput::QWheelEvent( wheelEvent4 ) );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, wheelEvent4.angleDelta().y() * 5 );
+  QCOMPARE( scene->cameraController()->mCameraBefore->viewCenter(), scene->cameraController()->cameraPose().centerPoint().toVector3D() );
 
   depthImage = Qgs3DUtils::captureSceneDepthBuffer( engine, scene );
-  grayImage = convertDepthImageToGray16Image( depthImage );
-  QGSVERIFYIMAGECHECK( "depth_wheel_action_4", "depth_wheel_action_4", grayImage, QString(), 550, QSize( 0, 0 ), 2 );
+  grayImage = convertDepthImageToGrayscaleImage( depthImage );
+  QGSVERIFYIMAGECHECK( "depth_wheel_action_4", "depth_wheel_action_4", grayImage, QString(), 5, QSize( 0, 0 ), 2 );
 
   scene->cameraController()->depthBufferCaptured( depthImage );
 
-  QGSCOMPARENEARVECTOR3D( testCam->zoomPoint(), QVector3D( -32.3, 221.7, 183.2 ), 1.0 );
-  QGSCOMPARENEARVECTOR3D( testCam->cameraPose()->centerPoint(), QVector3D( -32.3, 221.7, 183.2 ), 1.0 );
-  QGSCOMPARENEAR( testCam->cameraPose()->distanceFromCenterPoint(), 101.1, 0.1 );
-  QCOMPARE( testCam->cumulatedWheelY(), 0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->mZoomPoint, QVector3D( -32.3, 221.7, 183.2 ), 1.0 );
+  QGSCOMPARENEARVECTOR3D( scene->cameraController()->cameraPose().centerPoint(), QVector3D( -32.3, 221.7, 183.2 ), 1.0 );
+  QGSCOMPARENEAR( scene->cameraController()->cameraPose().distanceFromCenterPoint(), 101.1, 0.1 );
+  QCOMPARE( scene->cameraController()->mCumulatedWheelY, 0 );
 
   // Checking camera position
   QVector3D diff = scene->cameraController()->camera()->position() - startPos;
@@ -1736,7 +1776,7 @@ void TestQgs3DRendering::test3DSceneExporter()
   const QgsRectangle fullExtent = layerPoly->extent();
 
   // =========== create polygon 3D renderer
-  QgsPolygon3DSymbol *symbol3d = new QgsPolygon3DSymbol;
+  std::unique_ptr< QgsPolygon3DSymbol > symbol3d = std::make_unique< QgsPolygon3DSymbol >();
   symbol3d->setExtrusionHeight( 10.f );
 
   QgsPhongMaterialSettings materialSettings;
@@ -1770,15 +1810,15 @@ void TestQgs3DRendering::test3DSceneExporter()
   engine.setRootEntity( scene );
 
   // =========== check with 1 big tile ==> 1 exported object
-  do3DSceneExport( 1, 1, 165, scene, symbol3d, layerPoly, &engine );
+  do3DSceneExport( 1, 1, 165, scene, symbol3d.get(), layerPoly, &engine );
   // =========== check with 4 tiles ==> 3 exported objects
-  do3DSceneExport( 2, 1, 165, scene, symbol3d, layerPoly, &engine );
+  do3DSceneExport( 2, 1, 165, scene, symbol3d.get(), layerPoly, &engine );
   // =========== check with 9 tiles ==> 3 exported objects
-  do3DSceneExport( 3, 3, 165, scene, symbol3d, layerPoly, &engine );
+  do3DSceneExport( 3, 3, 165, scene, symbol3d.get(), layerPoly, &engine );
   // =========== check with 16 tiles ==> 3 exported objects
-  do3DSceneExport( 4, 3, 165, scene, symbol3d, layerPoly, &engine );
+  do3DSceneExport( 4, 3, 165, scene, symbol3d.get(), layerPoly, &engine );
   // =========== check with 25 tiles ==> 3 exported objects
-  do3DSceneExport( 5, 3, 165, scene, symbol3d, layerPoly, &engine );
+  do3DSceneExport( 5, 3, 165, scene, symbol3d.get(), layerPoly, &engine );
 
   delete scene;
   mapSettings.setLayers( {} );
