@@ -9,8 +9,6 @@ the Free Software Foundation; either version 2 of the License, or
 __author__ = 'Alessandro Pasotti'
 __date__ = '10/08/2019'
 __copyright__ = 'Copyright 2019, The QGIS Project'
-# This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '47373234acd430af1674b659785dd3d743b936f6'
 
 import os
 
@@ -87,7 +85,7 @@ class TestPyQgsProviderConnectionPostgres(unittest.TestCase, TestPyQgsProviderCo
         settings = QgsSettings()
         settings.beginGroup('/PostgreSQL/connections/my_sslmode_test')
         self.assertEqual(settings.value("sslmode"), 'SslVerifyCa')
-        self.assertEqual(settings.enumValue("sslmode", QgsDataSourceUri.SslPrefer), QgsDataSourceUri.SslVerifyCa)
+        self.assertEqual(settings.enumValue("sslmode", QgsDataSourceUri.SslMode.SslPrefer), QgsDataSourceUri.SslMode.SslVerifyCa)
 
     def test_postgis_geometry_filter(self):
         """Make sure the postgres provider only returns one matching geometry record and no polygons etc."""
@@ -114,39 +112,39 @@ class TestPyQgsProviderConnectionPostgres(unittest.TestCase, TestPyQgsProviderCo
 
         # Retrieve capabilities
         capabilities = conn.capabilities()
-        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Tables))
-        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Schemas))
-        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.CreateVectorTable))
-        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.DropVectorTable))
-        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.RenameVectorTable))
-        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.RenameRasterTable))
+        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Capability.Tables))
+        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Capability.Schemas))
+        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Capability.CreateVectorTable))
+        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Capability.DropVectorTable))
+        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Capability.RenameVectorTable))
+        self.assertTrue(bool(capabilities & QgsAbstractDatabaseProviderConnection.Capability.RenameRasterTable))
 
         # Check filters and special cases
-        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Raster))
-        self.assertTrue('Raster1' in table_names)
-        self.assertFalse('geometryless_table' in table_names)
-        self.assertFalse('geometries_table' in table_names)
-        self.assertFalse('geometries_view' in table_names)
+        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Raster))
+        self.assertIn('Raster1', table_names)
+        self.assertNotIn('geometryless_table', table_names)
+        self.assertNotIn('geometries_table', table_names)
+        self.assertNotIn('geometries_view', table_names)
 
-        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.View))
-        self.assertFalse('Raster1' in table_names)
-        self.assertFalse('geometryless_table' in table_names)
-        self.assertFalse('geometries_table' in table_names)
-        self.assertTrue('geometries_view' in table_names)
+        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.View))
+        self.assertNotIn('Raster1', table_names)
+        self.assertNotIn('geometryless_table', table_names)
+        self.assertNotIn('geometries_table', table_names)
+        self.assertIn('geometries_view', table_names)
 
-        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Aspatial))
-        self.assertFalse('Raster1' in table_names)
-        self.assertTrue('geometryless_table' in table_names)
-        self.assertFalse('geometries_table' in table_names)
-        self.assertFalse('geometries_view' in table_names)
+        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Aspatial))
+        self.assertNotIn('Raster1', table_names)
+        self.assertIn('geometryless_table', table_names)
+        self.assertNotIn('geometries_table', table_names)
+        self.assertNotIn('geometries_view', table_names)
 
-        tables = conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Aspatial | QgsAbstractDatabaseProviderConnection.View)
+        tables = conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Aspatial | QgsAbstractDatabaseProviderConnection.TableFlag.View)
         table_names = self._table_names(tables)
         b32523_view = self._table_by_name(tables, 'b32523')
         self.assertTrue(b32523_view)
         pks = b32523_view.primaryKeyColumns()
-        self.assertTrue('pk' in pks)
-        self.assertTrue('random' in pks)
+        self.assertIn('pk', pks)
+        self.assertIn('random', pks)
 
         geometries_table = self._table_by_name(conn.tables('qgis_test'), 'geometries_table')
         srids_and_types = [[t.crs.postgisSrid(), t.wkbType]
@@ -156,13 +154,13 @@ class TestPyQgsProviderConnectionPostgres(unittest.TestCase, TestPyQgsProviderCo
                          [[0, 1], [0, 2], [0, 3], [0, 7], [3857, 1], [4326, 1]])
 
         # Check TopoGeometry and Pointcloud layers are found in vector table names
-        tables = conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Vector)
+        tables = conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Vector)
         table_names = self._table_names(tables)
-        self.assertTrue('TopoLayer1' in table_names)
-        self.assertTrue('PointCloudPointLayer' in table_names)
-        self.assertTrue('PointCloudPatchLayer' in table_names)
+        self.assertIn('TopoLayer1', table_names)
+        self.assertIn('PointCloudPointLayer', table_names)
+        self.assertIn('PointCloudPatchLayer', table_names)
 
-        self.assertTrue('geometries_table' in table_names)
+        self.assertIn('geometries_table', table_names)
 
         # Revoke select permissions on topology.topology from qgis_test_user
         conn.executeSql('REVOKE SELECT ON topology.topology FROM qgis_test_user')
@@ -179,14 +177,14 @@ class TestPyQgsProviderConnectionPostgres(unittest.TestCase, TestPyQgsProviderCo
         newconn = md.createConnection(newuri, {})
 
         # Check TopoGeometry and Pointcloud layers are not found in vector table names
-        tableTypes = QgsAbstractDatabaseProviderConnection.Vector | QgsAbstractDatabaseProviderConnection.Raster
+        tableTypes = QgsAbstractDatabaseProviderConnection.TableFlag.Vector | QgsAbstractDatabaseProviderConnection.TableFlag.Raster
         tables = newconn.tables('qgis_test', tableTypes)
         table_names = self._table_names(tables)
-        self.assertFalse('TopoLayer1' in table_names)
-        self.assertFalse('PointCloudPointLayer' in table_names)
-        self.assertFalse('PointCloudPatchLayer' in table_names)
-        self.assertFalse('Raster1' in table_names)
-        self.assertTrue('geometries_table' in table_names)
+        self.assertNotIn('TopoLayer1', table_names)
+        self.assertNotIn('PointCloudPointLayer', table_names)
+        self.assertNotIn('PointCloudPatchLayer', table_names)
+        self.assertNotIn('Raster1', table_names)
+        self.assertIn('geometries_table', table_names)
 
         # TODO: only revoke select permission on topology.layer, grant
         #       on topology.topology
@@ -219,18 +217,18 @@ class TestPyQgsProviderConnectionPostgres(unittest.TestCase, TestPyQgsProviderCo
         conn = md.createConnection(self.uri, {})
         md.saveConnection(conn, 'qgis_test1')
 
-        table = self._table_by_name(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Raster), 'Raster1')
+        table = self._table_by_name(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Raster), 'Raster1')
         self.assertTrue(QgsRasterLayer(f"PG: {conn.uri()} dbname='qgis_test' schema='qgis_test' column='{table.geometryColumn()}' table='{table.tableName()}'", 'r1', 'gdal').isValid())
         conn.renameRasterTable('qgis_test', table.tableName(), 'Raster2')
-        table = self._table_by_name(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Raster), 'Raster2')
+        table = self._table_by_name(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Raster), 'Raster2')
         self.assertTrue(QgsRasterLayer(f"PG: {conn.uri()} dbname='qgis_test' schema='qgis_test' column='{table.geometryColumn()}' table='{table.tableName()}'", 'r1', 'gdal').isValid())
-        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Raster))
-        self.assertFalse('Raster1' in table_names)
-        self.assertTrue('Raster2' in table_names)
+        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Raster))
+        self.assertNotIn('Raster1', table_names)
+        self.assertIn('Raster2', table_names)
         conn.renameRasterTable('qgis_test', table.tableName(), 'Raster1')
-        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.Raster))
-        self.assertFalse('Raster2' in table_names)
-        self.assertTrue('Raster1' in table_names)
+        table_names = self._table_names(conn.tables('qgis_test', QgsAbstractDatabaseProviderConnection.TableFlag.Raster))
+        self.assertNotIn('Raster2', table_names)
+        self.assertIn('Raster1', table_names)
 
     def test_true_false(self):
         """Test returned values from BOOL queries"""
@@ -295,7 +293,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
 
         conn.executeSql(foreign_table_definition)
 
-        self.assertNotEqual(conn.tables('public', QgsAbstractDatabaseProviderConnection.Foreign | QgsAbstractDatabaseProviderConnection.Aspatial), [])
+        self.assertNotEqual(conn.tables('public', QgsAbstractDatabaseProviderConnection.TableFlag.Foreign | QgsAbstractDatabaseProviderConnection.TableFlag.Aspatial), [])
 
     @unittest.skipIf(os.environ.get('QGIS_CONTINUOUS_INTEGRATION_RUN', 'true'), 'Disabled on Travis')
     def test_foreign_table_server(self):
@@ -323,7 +321,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         INTO foreign_schema;
         """.format(host=host, user=user, port=port, dbname=dbname, password=password, service=service)
         conn.executeSql(foreign_table_definition)
-        self.assertEqual(conn.tables('foreign_schema', QgsAbstractDatabaseProviderConnection.Foreign)[0].tableName(), 'someData')
+        self.assertEqual(conn.tables('foreign_schema', QgsAbstractDatabaseProviderConnection.TableFlag.Foreign)[0].tableName(), 'someData')
 
     def test_fields(self):
         """Test fields"""
@@ -400,6 +398,26 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         table_info = conn.table('qgis_test', 'gh_43268_test_zm')
         self.assertEqual(sorted([QgsWkbTypes.displayString(col.wkbType) for col in table_info.geometryColumnTypes()]), ['LineStringZ', 'PointZ', 'PolygonZ'])
 
+    def test_m(self):
+        """Test regression GH #55223"""
+
+        md = QgsProviderRegistry.instance().providerMetadata('postgres')
+        conn = md.createConnection(self.uri, {})
+        sql = """
+        DROP TABLE IF EXISTS qgis_test.gh_55223_test_m;
+        CREATE TABLE qgis_test.gh_55223_test_m AS
+        WITH test_measure AS (
+            SELECT ST_GeomFromText('LINESTRING M (796091.2 6313188.3 0,796089 6313175 13.480726983172712,796087 6313166.1 22.602678529121157,796083.6 6313156.5 32.786980173957)', 2154) AS xym
+        )
+        select
+            xym,
+            ST_Force2D(xym) as xy
+        FROM test_measure;
+        """
+        conn.executeSql(sql)
+        geom_types = [t.geometryColumnTypes()[0] for t in conn.tables() if t.tableName() == 'gh_55223_test_m']
+        self.assertEqual(sorted([QgsWkbTypes.displayString(col.wkbType) for col in geom_types]), ['LineString', 'LineStringM'])
+
     def test_table_scan(self):
         """Test that with use estimated metadata disabled all geometry column
         types can be identified, test for GH #43186 """
@@ -443,13 +461,13 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         uri.setSchema('qgis_test')
         uri.setTable('geometry_table_with_multiple_types')
         uri.setGeometryColumn('geom')
-        uri.setWkbType(QgsWkbTypes.Point)
+        uri.setWkbType(QgsWkbTypes.Type.Point)
         vl = QgsVectorLayer(uri.uri(), 'points', 'postgres')
         self.assertTrue(vl.isValid())
         self.assertEqual(vl.featureCount(), 110)
 
         uri.setGeometryColumn('geom')
-        uri.setWkbType(QgsWkbTypes.LineString)
+        uri.setWkbType(QgsWkbTypes.Type.LineString)
         vl = QgsVectorLayer(uri.uri(), 'lines', 'postgres')
         self.assertTrue(vl.isValid())
         self.assertEqual(vl.featureCount(), 10)
@@ -481,7 +499,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         self.assertTrue(vl.isSqlQuery())
         # Test flags
         self.assertTrue(vl.vectorLayerTypeFlags() & Qgis.VectorLayerTypeFlag.SqlQuery)
-        self.assertEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 0)
 
@@ -489,7 +507,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         vl = conn.createSqlVectorLayer(options)
         self.assertTrue(vl.isValid())
         self.assertTrue(vl.isSqlQuery())
-        self.assertEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 2)
 
@@ -497,7 +515,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         vl = conn.createSqlVectorLayer(options)
         self.assertTrue(vl.isValid())
         self.assertTrue(vl.isSqlQuery())
-        self.assertEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 1)
 
@@ -506,7 +524,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         vl = conn.createSqlVectorLayer(options)
         self.assertTrue(vl.isValid())
         self.assertTrue(vl.isSqlQuery())
-        self.assertEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 1)
 
@@ -539,7 +557,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         vl = conn.createSqlVectorLayer(options)
         self.assertTrue(vl.isValid())
         self.assertTrue(vl.isSqlQuery())
-        self.assertNotEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertNotEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 1)
 
@@ -568,7 +586,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         # Test flags
         self.assertTrue(vl.vectorLayerTypeFlags() & Qgis.VectorLayerTypeFlag.SqlQuery)
 
-        self.assertEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 1)
 
@@ -578,7 +596,7 @@ CREATE FOREIGN TABLE IF NOT EXISTS points_csv (
         vl = conn.createSqlVectorLayer(options)
         self.assertTrue(vl.isSqlQuery())
         self.assertTrue(vl.isValid())
-        self.assertEqual(vl.geometryType(), QgsWkbTypes.PointGeometry)
+        self.assertEqual(vl.geometryType(), QgsWkbTypes.GeometryType.PointGeometry)
         features = [f for f in vl.getFeatures()]
         self.assertEqual(len(features), 1)
 
