@@ -16,7 +16,7 @@ import os
 import subprocess
 import tempfile
 
-from qgis.PyQt.QtCore import QRectF, QSize, Qt, QUuid
+from qgis.PyQt.QtCore import QRectF, QSize, Qt, QUuid, QCoreApplication
 from qgis.PyQt.QtGui import QColor, QImage, QPainter
 from qgis.core import (
     Qgis,
@@ -26,13 +26,16 @@ from qgis.core import (
     QgsLayoutItemMap,
     QgsLayoutItemPage,
     QgsLayoutSize,
+    QgsLineSymbol,
     QgsMapRendererCache,
     QgsMapRendererCustomPainterJob,
     QgsMapRendererParallelJob,
     QgsMapRendererSequentialJob,
     QgsMapSettings,
+    QgsMarkerLineSymbolLayer,
     QgsMarkerSymbol,
     QgsMaskMarkerSymbolLayer,
+    QgsNullSymbolRenderer,
     QgsOuterGlowEffect,
     QgsPalLayerSettings,
     QgsPathResolver,
@@ -48,7 +51,8 @@ from qgis.core import (
     QgsSymbolLayerUtils,
     QgsUnitTypes,
     QgsWkbTypes,
-    QgsFontUtils
+    QgsFontUtils,
+    QgsSettings
 )
 import unittest
 from qgis.testing import start_app, QgisTestCase
@@ -77,14 +81,9 @@ def renderMapToImageWithTime(mapsettings, parallel=False, cache=None):
     return (job.renderedImage(), job.renderingTime())
 
 
-class TestSelectiveMasking(QgisTestCase):
-
-    @classmethod
-    def control_path_prefix(cls):
-        return "selective_masking"
+class SelectiveMaskingTestBase():
 
     def setUp(self):
-
         self.map_settings = QgsMapSettings()
         crs = QgsCoordinateReferenceSystem('epsg:4326')
         extent = QgsRectangle(-123.0, 22.7, -76.4, 46.9)
@@ -178,7 +177,7 @@ class TestSelectiveMasking(QgisTestCase):
                     renderMapToImageWithTime(map_settings, parallel=do_parallel, cache=cache)
                 img, t = renderMapToImageWithTime(map_settings, parallel=do_parallel, cache=cache)
 
-                suffix = ("_parallel" if do_parallel else "_sequential") + ("_cache" if use_cache else "_nocache")
+                suffix = "_" + self.suite_id() + ("_parallel" if do_parallel else "_sequential") + ("_cache" if use_cache else "_nocache")
                 res = self.image_check(
                     control_name + suffix,
                     control_name,
@@ -241,7 +240,8 @@ class TestSelectiveMasking(QgisTestCase):
                             "-png", "-r", "300", "-singlefile"])
 
             rendered_image = QImage(image_result_filename)
-            res = self.image_check(control_name,
+            suffix = "_" + self.suite_id()
+            res = self.image_check(control_name + suffix,
                                    control_name,
                                    rendered_image,
                                    control_name,
@@ -1326,6 +1326,8 @@ class TestSelectiveMasking(QgisTestCase):
 
         sl = QgsSvgMarkerSymbolLayer(svgPath, 5)
         sl.setFillColor(QColor("blue"))
+        sl.setStrokeColor(QColor("white"))
+        sl.setStrokeWidth(0)
 
         p = QgsMarkerSymbol()
         p.changeSymbolLayer(0, sl)
@@ -1347,7 +1349,80 @@ class TestSelectiveMasking(QgisTestCase):
         # no rasters
         self.check_layout_export("layout_export_svg_marker_masking", 0, [self.points_layer, self.lines_layer])
 
+    def test_markerline_masked(self):
+        """
+        Test a layout export where a QgsMarkerLineSymbolLayer is masked
+        """
+
+        sl = QgsMarkerLineSymbolLayer(True, 7)
+        circle_symbol = QgsMarkerSymbol.createSimple({'size': '3'})
+        sl.setSubSymbol(circle_symbol)
+
+        symbol = QgsLineSymbol.createSimple({})
+        symbol.changeSymbolLayer(0, sl)
+        self.lines_layer.setRenderer(QgsSingleSymbolRenderer(symbol))
+        self.polys_layer.setRenderer(QgsNullSymbolRenderer())
+
+        label_settings = self.polys_layer.labeling().settings()
+        fmt = label_settings.format()
+        # enable a mask
+        fmt.mask().setEnabled(True)
+        fmt.mask().setSize(4.0)
+        # and mask other symbol layers underneath
+        fmt.mask().setMaskedSymbolLayers([QgsSymbolLayerReference(self.lines_layer.id(), sl.id())])
+        label_settings.setFormat(fmt)
+        self.polys_layer.labeling().setSettings(label_settings)
+
+        self.check_layout_export("layout_export_markerline_masked", 0, [self.polys_layer, self.lines_layer])
+
+
+class TestSelectiveMaskingQPainterPathBackend(QgisTestCase, SelectiveMaskingTestBase):
+    """
+    Test selective masking with the QPainterPath backend
+    """
+    @classmethod
+    def control_path_prefix(cls):
+        return "selective_masking"
+
+    @classmethod
+    def suite_id(cls):
+        return "qpainterpath"
+
+    @classmethod
+    def setUpClass(cls):
+        QgsSettings().setValue('map/mask-backend', 'qpainterpath')
+        QgisTestCase.setUpClass()
+
+    def setUp(self):
+        SelectiveMaskingTestBase.setUp(self)
+
+
+class TestSelectiveMaskingGeometryBackend(QgisTestCase, SelectiveMaskingTestBase):
+    """
+    Test selective masking with the QgsGeometry backend
+    """
+    @classmethod
+    def control_path_prefix(cls):
+        return "selective_masking"
+
+    @classmethod
+    def suite_id(cls):
+        return "geometry"
+
+    @classmethod
+    def setUpClass(cls):
+        QgsSettings().setValue('map/mask-backend', 'geometry')
+        QgisTestCase.setUpClass()
+
+    def setUp(self):
+        SelectiveMaskingTestBase.setUp(self)
+
 
 if __name__ == '__main__':
+    QCoreApplication.setOrganizationName("QGIS_Test")
+    QCoreApplication.setOrganizationDomain("SelectiveMaskingTestBase.com")
+    QCoreApplication.setApplicationName("SelectiveMaskingTestBase")
+    QgsSettings().clear()
+
     start_app()
     unittest.main()
